#include <tcl.h>
#include <tk.h>
#include "error.h"
#include "find.h"
#include "focus.h"
#include "map.h"
#include "misc.h"
#include "move.h"
#include "pointer.h"
#include "reparent.h"
#include "resize.h"
#include "send.button.press.h"
#include "send.button.release.h"
#include "send.key.press.h"
#include "send.key.release.h"
#include "send.string.h"
#include "delete.or.kill.h"

int Tkxext_Init (Tcl_Interp *interp) {
 Tk_Window tkwin;
 Display *dis;

 if (NULL == Tcl_InitStubs (interp, TCL_VERSION, 0)) 
  return TCL_ERROR;

 if (NULL == Tk_InitStubs (interp, TK_VERSION, 0))
  return TCL_ERROR;

 dis = Tk_Display (Tk_MainWindow (interp));

 TKXEXT_WM_PROTOCOLS = XInternAtom (dis, "WM_PROTOCOLS", 0);
 TKXEXT_WM_DELETE_WINDOW = XInternAtom (dis, "WM_DELETE_WINDOW", 0);

 Tk_CreateErrorHandler (dis, /*all error events*/ -1, 
   /*all request errors*/ -1, /*all minor_code errors*/ -1,
   TkXext_XErrorHandler, (ClientData) dis);

#define OBJ_CMD(cmdName,function) \
 Tcl_CreateObjCommand(interp,cmdName,function,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL)

 /* delete.or.kill.c */
 OBJ_CMD ("TkXext.delete.or.kill", TkXext_DeleteOrKill);

 /* find.c */
 OBJ_CMD ("TkXext.find.window", TkXext_FindWindow);
 
 /* focus.c */
 OBJ_CMD ("TkXext.focus", TkXext_Focus);

 /* map.c */
 OBJ_CMD ("TkXext.map.window", TkXext_MapWindow);
 OBJ_CMD ("TkXext.unmap.window", TkXext_UnmapWindow);

 /* misc.c */
 OBJ_CMD ("TkXext.make.window.exist", TkXext_MakeWindowExist); 

 /* move.c */
 OBJ_CMD ("TkXext.move.window", TkXext_MoveWindow);

 /* pointer.c */
 OBJ_CMD ("TkXext.move.pointer", TkXext_MovePointer);

 /* reparent.c */
 OBJ_CMD ("TkXext.reparent.window", TkXext_ReparentWindow);

 /* resize.c */
 OBJ_CMD ("TkXext.resize.window", TkXext_ResizeWindow);
 OBJ_CMD ("TkXext.get.resize.increments", TkXext_GetResizeIncrements);

 /* send.button.press.c */
 OBJ_CMD ("TkXext.send.button.press", TkXext_SendButtonPress);
 
 /* send.button.release.c */ 
 OBJ_CMD ("TkXext.send.button.release", TkXext_SendButtonRelease);

 /* send.key.press.c */
 OBJ_CMD ("TkXext.send.key.press", TkXext_SendKeyPress);

 /* send.key.release.c */
 OBJ_CMD ("TkXext.send.key.release", TkXext_SendKeyRelease);

 /* send.string.c */
 OBJ_CMD ("TkXext.send.string", TkXext_SendString);

 Tcl_PkgProvide (interp, "TkXext", EXT_VERSION);

 tkwin = Tk_MainWindow (interp);

 Tk_MakeWindowExist (tkwin);

 XTestGrabControl (Tk_Display (tkwin), /*impervious to grabs*/ 1);

 return TCL_OK;
}
