
#include "TkXext.h"
#include "string.to.window.h"

static int getArguments (
 Tcl_Interp *interp, Tcl_Obj *CONST objv[], int objc,
 Window *wPtr, Window *parentPtr, int *xPtr, int *yPtr
) {
 if (TCL_OK != TkXext_StringToWindow (interp, Tcl_GetString (objv[1]), wPtr)) {
  return TCL_ERROR;
 }

 if (TCL_OK != TkXext_StringToWindow (interp, Tcl_GetString (objv[2]), parentPtr)) {
  return TCL_ERROR;
 }
 
 if (5 == objc) {
  if (TCL_OK != Tcl_GetIntFromObj (interp, objv[3], xPtr)) {
   return TCL_ERROR;
  }
 
  if (TCL_OK != Tcl_GetIntFromObj (interp, objv[4], yPtr)) {
   return TCL_ERROR;
  }
 }
 return TCL_OK;
}

static int getWM_STATE (Display *dis, Window win) {
 int status = 0;
 long state = 0;
 static Atom aWM_STATE = 0;
 Atom actual_type = 0;
 int actual_format = 0;
 unsigned long num_ret = 0, num_left = 0;
 unsigned char *data = NULL;

 if (!aWM_STATE) {
  aWM_STATE = XInternAtom(dis, "WM_STATE", False);
 }

 status = XGetWindowProperty (dis,
  win,
  aWM_STATE,
  0L,
  sizeof (long),
  False,
  aWM_STATE,
  &actual_type, &actual_format,
  &num_ret, &num_left,
  &data
 );

 /*I should point out that Success is defined as 0 in X11*/
 if ((status == Success) && (NULL != data)) {
  state = *(long *)data;
 } else {
  state = -1;
 }

 if (data) {
  XFree (data);
 }
 return state;
}

static int windowExists (Tcl_Interp *interp, Display *dis, Window w) {
 XWindowAttributes xwattr;
 if (0 == XGetWindowAttributes (dis, w, &xwattr)) {
  Tcl_SetResult (interp, "unable to get window attributes (invalid Window/XID?)", TCL_STATIC);
  return 0;
 }
 return 1;
}

static void withdrawWindowFromDesktop (Display *dis, Window w, int screenNum) {
 int wm_state;

 XWithdrawWindow (dis, w, screenNum);
 XSync (dis, 0);
 while ((wm_state = getWM_STATE (dis, w)) != 0 && wm_state != -1) {
  XSync (dis, 0);
  XWithdrawWindow (dis, w, screenNum);
 }
}

int TkXext_ReparentWindow TCL_OBJ_CMD_ARGS {
 Tk_Window tkwin;
 Display *dis;
 Window newParent;
 Window w;
 int screenNum;
 int x = 0; 
 int y = 0;
 
 /*TkXext.reparent.window XID new-parent ?x ?y*/
 if (3 != objc && 5 != objc) {
  Tcl_WrongNumArgs (interp, 1, objv, "Window/XID new-parent-XID");
  return TCL_ERROR;
 } 
 
 tkwin = Tk_MainWindow (interp);
 Tk_MakeWindowExist (tkwin);
 dis = Tk_Display (tkwin);
 screenNum = Tk_ScreenNumber (tkwin);

 if (TCL_OK != getArguments (interp, objv, objc, &w, &newParent, &x, &y)) {
  return TCL_ERROR;
 }

 if (!windowExists(interp, dis, w)) {
  return TCL_ERROR;
 }
 
 withdrawWindowFromDesktop (dis, w, screenNum);
 
 XAddToSaveSet (dis, w);
 XReparentWindow (dis, w, newParent, x, y);
 XMapWindow (dis, w);
 XFlush (dis);

 return TCL_OK; 
}
