#include "TkXext.h"
#include "string.to.window.h"
 
int TkXext_ResizeWindow TCL_OBJ_CMD_ARGS {
 Tk_Window tkwin;
 int width;
 int height;
 char *winStr;
 Display *dis;
 Window w;

 /*TkXext.resize.window XID width height*/
 if (4 != objc) {
  Tcl_WrongNumArgs (interp, 1, objv, "XID width height");
  return TCL_ERROR;
 }

 tkwin = Tk_MainWindow (interp);
 Tk_MakeWindowExist (tkwin);
 dis = Tk_Display (tkwin);
 
 winStr = Tcl_GetString (objv[1]);
 
 if (TCL_OK != TkXext_StringToWindow (interp, winStr, &w)) {
  return TCL_ERROR;
 }
 
 if (TCL_OK != Tcl_GetIntFromObj (interp, objv[2], &width)) {
  return TCL_ERROR;
 }

 if (TCL_OK != Tcl_GetIntFromObj (interp, objv[3], &height)) {
  return TCL_ERROR;
 }
 
 XResizeWindow (dis, w, width, height);
 XFlush (dis); 

 return TCL_OK;
}

int TkXext_GetResizeIncrements TCL_OBJ_CMD_ARGS {
 XSizeHints sizeHints;
 Display *dis;
 Window w;
 char *winStr;
 unsigned long hintsFromUser;
 Tcl_Obj *result;
 Tk_Window tkwin;
 
 /*TkXext.get.resize.increments Window/XID*/
 if (2 != objc) {
  Tcl_WrongNumArgs (interp, 1, objv, "Window/XID");
  return TCL_ERROR;
 } 

 tkwin = Tk_MainWindow (interp);
 Tk_MakeWindowExist (tkwin);
 dis = Tk_Display (tkwin);
  
 winStr = Tcl_GetString (objv[1]);
 if (TCL_OK != TkXext_StringToWindow (interp, winStr, &w)) {
  return TCL_ERROR;
 }
    
 if (!XGetWMNormalHints (dis, w, &sizeHints, &hintsFromUser)) {
  /*
   * XXX we should probably set the resize increments to 1,1
   * rather than return TCL_ERROR.
   */

  Tcl_SetResult (interp, "unable to get window normal hints", TCL_STATIC);
  return TCL_ERROR;
 }
 
 result = Tcl_NewListObj (0, NULL);
 Tcl_IncrRefCount (result);
   
 if (hintsFromUser & PResizeInc) {
  Tcl_Obj *width;
  Tcl_Obj *height;
  
  width = Tcl_NewIntObj (sizeHints.width_inc);
  Tcl_IncrRefCount (width);
  height = Tcl_NewIntObj (sizeHints.height_inc);
  Tcl_IncrRefCount (height);
  
  if (TCL_OK != Tcl_ListObjAppendElement (interp, result, width)) {
   Tcl_DecrRefCount (result);
   Tcl_DecrRefCount (width);
   return TCL_ERROR;
  }
  
  if (TCL_OK != Tcl_ListObjAppendElement (interp, result, height)) {
   Tcl_DecrRefCount (result);
   Tcl_DecrRefCount (height); 
   return TCL_ERROR;
  }
  Tcl_DecrRefCount (width); 
  Tcl_DecrRefCount (height);
 }  

 Tcl_SetObjResult (interp, result);
 Tcl_DecrRefCount (result);

 return TCL_OK;
}
